/* Datei lagerbestand.c */
#include <stdio.h>

#define NAME_LEN (16+1)

enum kategorieEnum {SCHREIBWAREN, DROGERIE, SONSTIGE};

typedef struct artikelTyp
{
   unsigned int nr;                 /* Eindeutige Artikelnummer */
   enum kategorieEnum kategorie;    /* Kategorie des Artikels */ 
   unsigned int anz;                /* Anzahl auf Lager */
   double preis;                    /* Preis pro Stueck */
   char name[NAME_LEN];             /* Name des Artikels */	
} artikelTyp;


void artikel_print (FILE * f, const artikelTyp * const art);

void artikel_printArray (FILE * f, artikelTyp arts[],
                         const unsigned int num);

void artikel_printKategorienLagerWerte (
        FILE * f,          /* Dateideskriptor, auf dem   */
                           /* ausgegeben wird            */
        artikelTyp arts[], /* Array mit Artikeln         */
        const unsigned int num  
                           /* Anzahl der Artikel in arts */
                                       );

int main (void)
{
   artikelTyp a[] = {{1271, SCHREIBWAREN, 2, 100., "Cascardi Fueller"},
                     {4542, DROGERIE, 200, 10., "LaBelle Seife"},
                     {5063, DROGERIE, 100,  3., "R-Tips"},
                     {5564, SONSTIGE, 400,  1., "Cq Banane"}
                    };
   artikel_printArray (stdout, a, sizeof(a)/sizeof(artikelTyp));

   {
      FILE * zFile;
      const char * fileName = "lagerKapital.lst"; /* Beachte doppelten \ */
      zFile = fopen(fileName, "w");
      if(!zFile)
      {
         fprintf(stderr, "Kann Datei %s nicht zum Schreiben oeffnen!\n", 
               fileName);
      }
      else
      {
         artikel_printKategorienLagerWerte(zFile, a,                                                                  
                  sizeof(a)/sizeof(artikelTyp) );
         fclose(zFile);
      }
   }

   return 0;
}

void artikel_print (FILE * f, const artikelTyp * const art)
{
   fprintf (f, "%d\n", art->nr);
   fprintf (f, "%d\n", art->kategorie);
   fprintf (f, "%d\n", art->anz);
   fprintf (f, "%f\n", art->preis);
   fprintf (f, "%s\n", art->name);
}

void artikel_printArray (FILE * f, artikelTyp arts[],
                         const unsigned int num)
{
   unsigned int i;
   for(i = 0; i < num; i++)
   {
      artikel_print (f, &arts[i]);
   }
}

void artikel_printKategorienLagerWerte(
         FILE * f, 			   /* Dateideskriptor, auf dem ausgegben wird */
         artikelTyp  arts[], 	/* Array mit Artikeln */
         const unsigned int num 	/* Anzahl der Artikel in arts */)
{
   enum kategorieEnum iKategorie;/* Laufende Kategorie-Nummer */
   unsigned int iArtikel;	/* Laufender Artikel in arts */
   for(iKategorie=0; iKategorie<=SONSTIGE; iKategorie++) 
   { /* Fr alle Kategorien ...*/
      double sum=0.0;						/* Summe je laufender Kategorie */
      for(iArtikel=0; iArtikel<num; iArtikel++)
      { /* Fr alle Artikel */
         const artikelTyp * const lfd = &arts[iArtikel];
         if(lfd->kategorie == iKategorie)
            sum += lfd->anz * lfd->preis;
         } /* Fr alle Artikel */

         /* Ausgabe: */
         fprintf(f, "Summe Kategorie Nr. %4d: %12.2f\n", 
                  iKategorie, sum );
   } /* Fr alle Kategorien ...*/
}

/*

a) Wozu dienen die zwei Typ-Attribute const beim zweiten Parameter?

Der erste const stellt sicher, dass keine Aenderungen an dem Artikel art
vorgenommen werden knnen.

Der zweite const stellt sicher, dass der Zeiger nicht in der Funktion
verschoben/geaendert wird und somit ein falscher Artikel ausgegeben wird.

b) Wie kann mit Hilfe des sizeof-Operators das gefhrliche Festlegen der Anzahl
der Elemente im Array a durch die literale Konstante 3 vermieden werden?

artikel_printArray(stdout, a, sizeof(a)/sizeof(artikelTyp) );

*/

