// DATEI: sudoku.c
/* Loesung eines 9x9 Sudokus mit Hilfe von Backtracking. */
#include <stdio.h>
#include <math.h>

/** Moegliche Ergebnisse bei einer Suche. */
enum suchErgebnis {SCHON_DA,      /**< Wert schon vorhanden */
				   NOCH_NICHT_DA, /**< Wert noch nicht vorhanden*/	
				  }; 	
/** Wie gross ist ein Block (i.d.R. 3) im Sudoku.
	@warning Es reicht nicht aus, einfach diese Zahl 
	zum Beispiel fuer 16x16 Sudokus hochzusetzen. Da
	ist einiges mehr zu tun...
	@todo UEBUNGSAUFGABE: Bauen Sie diese Loesung so um,
	dass 16x16 Sudokus mit den "Ziffern" 0..9 und A..F
	(also den Hexadezimalziffern) funktioniert.
*/
#define BLOCKBR 3

/** Anzahl der Spalten */
#define SPALTEN BLOCKBR*BLOCKBR

/** Anzahl der Zeilen */
#define ZEILEN SPALTEN

/** Zweidimensionales Array, welches das 
    initiale (und das geloeste) Sudoku speichert. 

	Moegliche Eintraege:
	@li Zahlen 1..9 
	@li ' ' fuer unbekannter Wert

	Es wird mit Strings (char *) gearbeitet,
	was zwar ein Zeichen mehr je Zeile 
	benoetigt ('\\0' am Ende), aber wesentlich leichter 
	mit dem Editor	eingebbar ist, wenn man ein neues 
	Raetsel aus der	Zeitung abtippt.
*/
char  zellen[ZEILEN][SPALTEN+1] = {
/*   --+--+--+	*/
	"   542   ",
	"967      ",
	"      318",
/*   --+--+--+	*/
	"    7 864",
	" 2 6 4 9 ",
	"645 1    ",
/*   --+--+--+	*/
	"891      ",
	"      547",
	"   326   ",
};

char  sudoku_sourceforge_net[ZEILEN][SPALTEN+1] = {
/*   --+--+--+	*/
	"   542   ",
	"967      ",
	"      318",
/*   --+--+--+	*/
	"    7 864",
	" 2 6 4 9 ",
	"645 1    ",
/*   --+--+--+	*/
	"891      ",
	"      547",
	"   326   ",
};

char  wikipedia[ZEILEN][SPALTEN+1] = {
/*   --+--+--+	*/
	"53  7    ",
	"6  195   ",
	" 98    6 ",
/*   --+--+--+	*/
	"8   6   3",
	"4  8 3  1",
	"7   2   6",
/*   --+--+--+	*/
	" 6    28 ",
	"   419  5",
	"    8  79",
};

/** Pruefen, ob ein gegebener Wert in einer Zeile erlaubt ist. 
  @param[in] zeile Zeile, in der der Wert geprueft wird 
  @param[in] wert  Wert, der noch nicht in der Zeile vorhanden sein darf
  @return NOCH_NICHT_DA, falls Wert noch nicht vorhanden.
  */
enum suchErgebnis isZeileOK (const unsigned int zeile, const char wert)
{
	unsigned int iSpalte;
	for (iSpalte=0; iSpalte < SPALTEN; iSpalte++)
		if (wert == zellen[zeile][iSpalte])
			return SCHON_DA;

	return NOCH_NICHT_DA;
}

/** Pruefen, ob ein gegebener Wert in einer Spalte erlaubt ist. 
  @param[in] spalte Spalte, in der der Wert geprueft wird                 
  @param[in] wert Wert, der noch nicht in der Spalte vorhanden sein darf
  @return NOCH_NICHT_DA, falls Wert noch nicht vorhanden.
  */
enum suchErgebnis isSpalteOK (const unsigned int spalte, const char wert)			 
{
	unsigned int iZeile;
	for (iZeile=0; iZeile < ZEILEN; iZeile++)
		if (wert == zellen[iZeile][spalte])
			return SCHON_DA;

	return NOCH_NICHT_DA;
}

/** Pruefen, ob ein gegebener Wert in einem Block erlaubt ist. 
  @param[in] zeile Zeile des Blocks, in der der Wert geprueft wird
  @param[in] spalte Spalte des Blocks, in der der Wert geprueft wird
  @param[in] wert Wert, der noch nicht in dem Block vorhanden sein darf
  @return NOCH_NICHT_DA, falls Wert noch nicht vorhanden.
  */
enum suchErgebnis isBlockOK (const unsigned int zeile,  
		const unsigned int spalte, 
		const char wert			)
{
	unsigned int iZeile, iSpalte;
	unsigned int iZeileStart  = (zeile/ BLOCKBR)*BLOCKBR;
	unsigned int iSpalteStart = (spalte/BLOCKBR)*BLOCKBR;

	for (iZeile = iZeileStart; iZeile < iZeileStart + BLOCKBR; iZeile++)
		for (iSpalte = iSpalteStart; iSpalte < iSpalteStart + BLOCKBR; iSpalte++)
			if (wert == zellen[iZeile][iSpalte])
				return SCHON_DA;

	return NOCH_NICHT_DA;
}
/** Ein Sudoku pruefen, ob keine Werte doppelt in Zeilen, Spalten oder Bloecken auftauchen. 
  Falls doch, einfach ausgeben, wo das der Fall ist.
  */
void printIfSudokuIsNotOK ()
{
	unsigned int iZeile, iSpalte;
	for (iZeile = 0; iZeile < ZEILEN; iZeile++)
	{
		for (iSpalte = 0; iSpalte < SPALTEN; iSpalte++)
		{
			char c = zellen[iZeile][iSpalte];
			if ( ' ' == c ) continue;
			zellen[iZeile][iSpalte] = ' ';
			if ( SCHON_DA == isZeileOK(iZeile, c) )
				printf ("Doppelter Zeilenwert %c in Zeile %u, Spalte %u", c, iZeile, iSpalte);
			if ( SCHON_DA == isSpalteOK (iSpalte, c) )
				printf ("Doppelter Spaltenwert %c in Zeile %u, Spalte %u", c, iZeile, iSpalte);
			if ( SCHON_DA == isBlockOK (iZeile, iSpalte, c) )
				printf ("Doppelter Blockwert %c in Zeile %u, Spalte %u", c, iZeile, iSpalte); 
			zellen[iZeile][iSpalte] = c;
		}
	}
}

/** Ein Sudoku ausgeben. */
void printSudoku ()
{
	unsigned int iZeile, iSpalte;
	const char * const th =  " + ------ + ------ + ------ +\n";

	printf("\n");
	for (iZeile = 0; iZeile < ZEILEN; iZeile++)
	{
		if (0 == iZeile % 3) printf(th); 
		for (iSpalte = 0; iSpalte < SPALTEN; iSpalte++)
		{
			if (0 == iSpalte % 3) printf(" | "); 
			printf("%c ", zellen[iZeile][iSpalte]);
		}
		printf(" |\n");
	}
	printf(th); 
	printIfSudokuIsNotOK();
}

/** Suche Loesungen des Sudokus.

  Es wird oben links beginnend versucht, alle Zahlen fuer ein freies Feld
  zu finden, die weder in der Zeile, noch in der Spalte, noch im Block
  doppelt auftauchen.  Falls es eine solche Zahl gibt, wird mit der naechsten
  Zelle weitergemacht, ansonsten wird die Suche abgebrochen
  (Backtracking) und in der aufrufenden 'suche' die naechste moegliche
  Zahl ausprobiert.

  @param zeile Startzeile der Suche
  @param spalte Startspalte der Suche
  */
void suche (const unsigned int zeile, const unsigned int spalte) 
{  
	static unsigned int nLsg  = 0; /* Anzahl der gefundenen Loesungen */
	static unsigned int nTestInsg = 0; /* Gesamtzahl der Tests */
	if (zeile >= ZEILEN) /* Loesung gefunden? */
	{
		printf ("\n%u. Loesung gefunden, insg. %u Tests:", ++nLsg, nTestInsg); 
		printSudoku();
	}
	else /* Weitersuchen bis einschl. zur letzten ((9,9)-ten) Zelle: */
	{
		/* Naechste u.U. zu testende Zelle bestimmen: */
		unsigned int iNextZeile, iNextSpalte;
		if ( spalte < SPALTEN - 1) /* Zeile noch nicht voll */
		{
			iNextSpalte = spalte + 1; iNextZeile = zeile;
		}
		else /* Zeile voll */
		{
			iNextSpalte = 0; iNextZeile = zeile + 1;
		}

		if (zellen [zeile][spalte] != ' ') 
		{  /* Zelle nicht frei, d.h. durch Raetsel vorbelegt */                                                         
			suche (iNextZeile, iNextSpalte);
		}
		else /* Zelle frei ... */
		{	 /* Alle Zahlen ausprobieren... */
			char wert;
			nTestInsg++;
			for (wert = '1'; wert <= '9'; wert++)
				if ( /* Wert ist mglich... */
						NOCH_NICHT_DA == isZeileOK  (zeile,         wert) &&
						NOCH_NICHT_DA == isSpalteOK (       spalte, wert) &&
						NOCH_NICHT_DA == isBlockOK  (zeile, spalte, wert)
				   )
				{
					zellen[zeile][spalte] = wert;
					suche (iNextZeile, iNextSpalte);
				}

			zellen[zeile][spalte] = ' ';
			return; 
		}
	}
}

int main(void)
{
	printSudoku();
	suche (0, 0);
	return 0;
}
